@@ -5,7 +5,7 @@
 # ColorDiff - a wrapper/replacment for 'diff' producing                #
 #             colourful output                                         #
 #                                                                      #
-# Copyright (C)2002-2009 Dave Ewart (davee@sungate.co.uk)              #
+# Copyright (C)2002-2012 Dave Ewart (davee@sungate.co.uk)              #
 #                                                                      #
 ########################################################################
 #                                                                      #
@@ -26,13 +26,14 @@ use Getopt::Long qw(:config pass_through);
 use IPC::Open2;
 
 my $app_name     = 'colordiff';
-my $version      = '1.0.10';
+my $version      = '1.0.11';
 my $author       = 'Dave Ewart';
 my $author_email = 'davee@sungate.co.uk';
 my $app_www      = 'http://colordiff.sourceforge.net/';
 my $copyright    = '(C)2002-2012';
 my $show_banner  = 1;
 my $color_patch  = 0;
+my $diff_cmd     = "diff";
 
 # ANSI sequences for colours
 my %colour;
@@ -69,6 +70,18 @@ my @config_files = ("$etcdir/colordiffrc");
 push (@config_files, "$ENV{HOME}/.colordiffrc") if (defined $ENV{HOME});
 my $config_file;
 
+# Convert tabs to spaces
+sub expand_tabs_to_spaces ($) {
+    my ($s) = @_;
+    while ((my $i = index ($s, "\t")) > -1) {
+        substr (
+            $s, $i, 1,    # range to replace
+            (' ' x (8 - ($i % 8))),    # string to replace with
+            );
+    }
+    $s;
+}
+
 sub check_for_file_arguments {
     my $nonopts = 0;
     my $ddash = 0;
@@ -114,6 +127,10 @@ foreach $config_file (@config_files) {
                 }
                 next;
             }
+            if ($setting eq 'diff_cmd') {
+                $diff_cmd = $value;
+                next;
+            }
             $setting =~ tr/A-Z/a-z/;
             $value   =~ tr/A-Z/a-z/;
             if (($value eq 'normal') || ($value eq 'none')) {
@@ -199,7 +216,7 @@ my @inputstream;
 my $exitcode = 0;
 if ($operating_methodology == 1) {
     # Run diff and then post-process the output
-    my $pid = open2(\*INPUTSTREAM, undef, "diff", @ARGV);
+    my $pid = open2(\*INPUTSTREAM, undef, "$diff_cmd", @ARGV);
     @inputstream = <INPUTSTREAM>;
     close INPUTSTREAM;
     waitpid $pid, 0;
@@ -225,9 +242,9 @@ DIFF_TYPE: foreach $record (@inputstream) {
         $diff_type = $specified_difftype;
         last DIFF_TYPE;
     }
-    # Unified diffs are the only flavour having '+++' or '---'
+    # Unified diffs are the only flavour having '+++ ' or '--- '
     # at the start of a line
-    if ($record =~ /^(\+\+\+|---|@@)/) {
+    if ($record =~ /^(\+\+\+ |--- |@@ )/) {
         $diff_type = 'diffu';
         last DIFF_TYPE;
     }
@@ -251,13 +268,17 @@ DIFF_TYPE: foreach $record (@inputstream) {
         last DIFF_TYPE;
     }
     # wdiff deleted/added patterns
-    # should almost always be pairwaise?
-    elsif ($record =~ /\[-.*?-\]/s) {
+    # should almost always be pairwise?
+    elsif ($record =~ /\[-.*?-\]/s
+           || $record =~ /\{\+.*?\+\}/s) {
         $diff_type = 'wdiff';
         last DIFF_TYPE;
     }
-    elsif ($record =~ /\{\+.*?\+\}/s) {
-        $diff_type = 'wdiff';
+    # FIXME - This is a bit risky, but if we haven't matched any other
+    # diff type by this stage, this line usually indicates we have
+    # debdiff output
+    elsif ($record =~ /^Control files: lines which differ/) {
+        $diff_type = 'debdiff';
         last DIFF_TYPE;
     }
 }
@@ -280,14 +301,7 @@ if ($diff_type eq 'diffy') {
     # Not very elegant, but does the job
     # Unfortunately requires parsing the input stream multiple times
     foreach (@inputstream) {
-        # Convert tabs to spaces
-        while ((my $i = index ($_, "\t")) > -1) {
-            substr (
-                $_, $i, 1,    # range to replace
-                (' ' x (8 - ($i % 8))),    # string to replace with
-            );
-        }
-        $record = $_;
+        $record = expand_tabs_to_spaces $_;
         $longest_record = length ($record) if (length ($record) > $longest_record);
     }
     for (my $i = 0 ; $i <= $longest_record ; $i++) {
@@ -296,25 +310,15 @@ if ($diff_type eq 'diffy') {
     }
 
     foreach (@inputstream) {
-        # Convert tabs to spaces
-        while ((my $i = index ($_, "\t")) > -1) {
-            substr (
-                $_, $i, 1,    # range to replace
-                (' ' x (8 - ($i % 8))),    # string to replace with
-            );
-        }
+        $_ = expand_tabs_to_spaces $_;
         for (my $i = 0 ; $i < (length ($_) - 2) ; $i++) {
             next if (!defined $separator_col{$i});
             next if ($separator_col{$i} == 0);
             my $subsub = substr ($_, $i, 2);
-            if (   ($subsub ne '  ')
-                && ($subsub ne ' |')
-                && ($subsub ne ' >')
-                && ($subsub ne ' <'))
-            {
+            if ($subsub !~ / [ |<>]/) {
                 $separator_col{$i} = 0;
             }
-            if (($subsub eq ' |') || ($subsub eq ' >') || ($subsub eq ' <')) {
+            if ($subsub =~ / [|<>]/) {
                 $candidate_col{$i}++;
             }
         }
@@ -440,12 +444,12 @@ foreach (@inputstream) {
         }
     }
     elsif ($diff_type eq 'wdiff') {
-        $_ =~ s/(\[-[^]]*?-\])/$file_old$1$colour{off}/g;
-        $_ =~ s/(\{\+[^]]*?\+\})/$file_new$1$colour{off}/g;
+        $_ =~ s/(\[-.+?-\])/$file_old$1$colour{off}/g;
+        $_ =~ s/(\{\+.+?\+\})/$file_new$1$colour{off}/g;
     }
     elsif ($diff_type eq 'debdiff') {
-        $_ =~ s/(\[-[^]]*?-\])/$file_old$1$colour{off}/g;
-        $_ =~ s/(\{\+[^]]*?\+\})/$file_new$1$colour{off}/g;
+        $_ =~ s/(\[-.+?-\])/$file_old$1$colour{off}/g;
+        $_ =~ s/(\{\+.+?\+\})/$file_new$1$colour{off}/g;
     }
     s/$/$colour{off}/;
     print "$_";
